#include "./ui_rps.h"

#include "RPS.h"
#include "Choice.h"
#include "Engine.h"
#include "AlgorithmFactory.h"
#include "GUIDisplay.h"
#include "TextDisplay.h"
#include "Who.h"

RPS::RPS(QWidget *parent)
    : QMainWindow(parent)
    , ui(new Ui::RPS)
{
    ui->setupUi(this);
    initialize_UI();

    // Seed the random number generator.
    srand(time(0));

    // Players.
    human    = new Human(Who::HUMAN);
    computer = new Computer(Who::COMPUTER);

    // Choice algorithms.
    human_algorithm = computer_algorithm = NULL;

    // Display and engine.
    Display *display = new GUIDisplay(this);
//    Display *display = new TextDisplay();
    engine = new Engine(human, computer, display);
}

RPS::~RPS()
{
    delete ui;

    delete human_algorithm;
    if (computer_algorithm != NULL) delete computer_algorithm;

    delete display;
    delete engine;
    delete human;
    delete computer;
}

void RPS::initialize_UI()
{
    // Choice algorithm menu.
    smart_index  = static_cast<int>(AlgorithmType::SMART);
    random_index = static_cast<int>(AlgorithmType::RANDOM);
    ui->algorithms_menu->addItem(
        QString(ALGORITHM_STRINGS[smart_index].c_str()),  smart_index);
    ui->algorithms_menu->addItem(
        QString(ALGORITHM_STRINGS[random_index].c_str()), random_index);

    // Max rounds spinner.
    ui->rounds->setValue(20);

    // Start button.
    ui->start_button->setEnabled(true);

    // Rock, paper, and scissors buttons.
    ui->rock_button->setEnabled(false);
    ui->paper_button->setEnabled(false);
    ui->scissors_button->setEnabled(false);
}

void RPS::on_start_button_clicked()
{
    // Select the choice algorithm from the menu.
    QVariant choice_data = ui->algorithms_menu->currentData();
    AlgorithmType algorithm_type = choice_data == smart_index
                                       ? AlgorithmType::SMART
                                       : AlgorithmType::RANDOM;

    // Create the choice algorithms.
    //    human_algorithm = AlgorithmFactory::make(AlgorithmType::BUTTON);
    human_algorithm = AlgorithmFactory::make(AlgorithmType::BUTTON);
//    human_algorithm = AlgorithmFactory::make(AlgorithmType::PROMPT);
    computer_algorithm = AlgorithmFactory::make(algorithm_type);

    human_algorithm->set_engine(engine);
    computer_algorithm->set_engine(engine);

    human->set_algorithm(human_algorithm);
    computer->set_algorithm(computer_algorithm);

    // Start a game.
    int max_rounds = ui->rounds->value();
    engine->start_game(max_rounds);
    engine->next_round();

//    engine->do_rounds();
}

void RPS::on_rock_button_clicked()
{
    do_round(Choice::ROCK);
}

void RPS::on_paper_button_clicked()
{
    do_round(Choice::PAPER);
}

void RPS::on_scissors_button_clicked()
{
    do_round(Choice::SCISSORS);
}

void RPS::do_round(Choice choice)
{
    engine->set_human_choice(choice);
    engine->do_round();
    engine->next_round();
}

void RPS::show_round(const int round) const
{
    ui->round_number->setText(QString(to_string(round).c_str()));
}

void RPS::show_start_game()
{
    // Disable the menu, spinner, and start button.
    ui->algorithms_menu->setEnabled(false);
    ui->rounds->setEnabled(false);
    ui->start_button->setEnabled(false);

    // Enable the rock, paper, and scissors buttons.
    ui->rock_button->setEnabled(true);
    ui->paper_button->setEnabled(true);
    ui->scissors_button->setEnabled(true);

    // Clear the choices and the winner.
    ui->human_choice->clear();
    ui->predicted_choice->clear();
    ui->computer_choice->clear();
    ui->winner->clear();

    // Zero out the wins and ties.
    ui->human_wins->setText("0");
    ui->computer_wins->setText("0");
    ui->ties->setText("0");
}

void RPS::show_end_game()
{
    // Enable the menu, spinner, and start button.
    ui->start_button->setEnabled(true);
    ui->algorithms_menu->setEnabled(true);
    ui->rounds->setEnabled(true);

    // Disable the rock, paper, and scissors buttons.
    ui->rock_button->setEnabled(false);
    ui->paper_button->setEnabled(false);
    ui->scissors_button->setEnabled(false);

    delete computer_algorithm;
    computer_algorithm = NULL;
}

void RPS::update_UI(const Choice human_choice,
                    const Choice computer_choice,
                    const Choice predicted_choice,
                    const Who winner,
                    const int human_wins,
                    const int computer_wins,
                    const int ties)
{
    // Display the choices and the winner.
    ui->human_choice->setText(QString(to_string(human_choice).c_str()));
    ui->predicted_choice->setText(QString(to_string(predicted_choice).c_str()));
    ui->computer_choice->setText(QString(to_string(computer_choice).c_str()));
    ui->winner->setText(QString(to_string(winner).c_str()));

    // Display the wins and ties.
    ui->human_wins->setText(QString(to_string(human_wins).c_str()));
    ui->computer_wins->setText(QString(to_string(computer_wins).c_str()));
    ui->ties->setText(QString(to_string(ties).c_str()));
}

